%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function:         [FVr_bestmem,S_bestval,I_nfeval] = deopt(fname,S_struct)
% Author:           Rainer Storn, Ken Price, Arnold Neumaier, Jim Van Zandt
% Modified by FLC \GECAD 04/winter/2017
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GECAD - GECCO and CEC 2020 Competition: Evolutionary Computation in Uncertain Environments: A Smart Grid Application 
%
% TEAM UC/CISECE-UT3/UCLV
% CUMDANCauchy-C1: a Cellular EDA Designed to Solve the Energy Resource Management Problem Under Uncertainty
%
%
% Phd Student Yoan Martinez Lopez, yoan.martinez@reduc.edu.cu [1,3]
% Phd Julio Madera, julio.madera@reduc.edu.cu [1]
% BSc Student Alexis Moya, alextkmoya@gmail.com [1]
% BSc Student Miguel Bethencourt Mayedo, miguel.bethencourt@reduc.edu.cu [1]
% Phd Ansel Y. Rodriguez-Gonzalez, ansel@cicese.mx [2]
%
%
% [1] UC (Universidad de Camaguey, Cuba)
% [2] CISECE-UT3 (Unidad de Transferencia Tecnolgica Tepic del Centro de Investigacin Cientfica y de Educacin Superior de Ensenada, Mexico)
% [3] UCLV (Universidad de Central de las Villas, Cuba)
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Fit_and_p,FVr_bestmemit, fitMaxVector] = ...
    edaNC(deParameters,caseStudyData,otherParameters,low_habitat_limit,up_habitat_limit,xbest,nEvals)

%-----This is just for notational convenience and to keep the code uncluttered.--------
I_NP         = deParameters.I_NP;
%F_weight     = deParameters.F_weight;
%F_CR         = deParameters.F_CR;
I_D          = numel(up_habitat_limit); %Number of variables or dimension
deParameters.nVariables=I_D;
FVr_minbound = low_habitat_limit;
FVr_maxbound = up_habitat_limit;
I_itermax    = round(deParameters.I_itermax);

%Repair boundary method employed
BRM=deParameters.I_bnd_constr; %1: bring the value to bound violated
                               %2: repair in the allowed range

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%I_strategy   = deParameters.I_strategy; %important variable
fnc= otherParameters.fnc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%-----Check input variables---------------------------------------------
if (I_NP < 5)
    I_NP=5;
   fprintf(1,' I_NP increased to minimal value 5\n');
end
%if ((F_CR < 0) || (F_CR > 1))
 %  F_CR=0.5;
  % fprintf(1,'F_CR should be from interval [0,1]; set to default value 0.5\n');
%end
if (I_itermax <= 0)
   I_itermax = 500;
   fprintf(1,'I_itermax should be > 0; set to default value 200\n');
end

%-----Initialize population and some arrays-------------------------------
%FM_pop = zeros(I_NP,I_D); %initialize FM_pop to gain speed
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% pre-allocation of loop variables
fitMaxVector = nan(1,I_itermax);
% limit iterations by threshold
gen = 1; %iterations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%----FM_pop is a matrix of size I_NPx(I_D+1). It will be initialized------
%----with random values between the min and max values of the-------------
%----parameters-----------------------------------------------------------
% FLC modification - vectorization
minPositionsMatrix=repmat(FVr_minbound,I_NP,1);
maxPositionsMatrix=repmat(FVr_maxbound,I_NP,1);
deParameters.minPositionsMatrix=minPositionsMatrix;
deParameters.maxPositionsMatrix=maxPositionsMatrix;

subp = deParameters.subpopulation;
selectInd = deParameters.selectInd;
elitist = deParameters.elitist;

I_itermax    =  floor((50000 - nEvals)/(deParameters.step))-1;%deParameters.stop;
 
% generate initial population.
rand('state',otherParameters.iRuns) %Guarantee same initial population
FM_pop=genpop(I_NP,I_D,minPositionsMatrix,maxPositionsMatrix);
FM_pop(1,:)=xbest;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%------Evaluate the best member after initialization----------------------
% Modified by FLC
%switch fnc
  %  case 'fitnessFun_DER'
  %  caseStudyData=caseStudyData(1);
%end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%------Evaluate the best member after initialization----------------------
[solFitness_M, ~,~]=feval(fnc,FM_pop,caseStudyData,otherParameters);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[S_val, ~] = max(solFitness_M,[],2); 
list  = sortrows([ (1:I_NP)' S_val(:, 1) ], 2);

 I_best_index = list(1:selectInd,1);
 sbest = mean(list(1:selectInd,2));
 FVr_bestmemit = mean(FM_pop(I_best_index,:)); % best member of current iteration

fitMaxVector(1,gen) = sbest;
 
%gen=gen+1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% store other information

%------EDA-Minimization---------------------------------------------
%------FM_popold is the population which has to compete. It is--------
%------static through one iteration. FM_pop is the newly--------------
%------emerging population.----------------------------------------
%FVr_rot  = (0:1:I_NP-1);               % rotating index array (size I_NP)
index_sub = list(1:subp,1);
FM_subpop = FM_pop(index_sub,:);

while gen < I_itermax %%&&  fitIterationGap >= threshold
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %[FM_ui,FM_base,~]=generate_trial(I_strategy,F_weight, F_CR, FM_pop, FVr_bestmemit,I_NP, I_D, FVr_rot);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    FM_ui = learningEDA(FM_subpop,I_NP);

    %% Boundary Control
    FM_ui = update_eda(FM_ui,minPositionsMatrix,maxPositionsMatrix,BRM);  
    
    %Evaluation of new Pop
    [solFitness_M_temp, ~,~]=feval(fnc,FM_ui,caseStudyData, otherParameters);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [S_val_temp, ~]=max(solFitness_M_temp,[],2); 
    list  = sortrows([ (1:I_NP)' S_val_temp(:, 1) ], 2);
     
    I_temp_index = list(1:selectInd,1);
    S_bestval = mean(list(1:selectInd,2));
   
    %% update best results
    
    if  (S_bestval) < (sbest)
      sbest = S_bestval;
      I_best_index = I_temp_index;
      FVr_bestmemit = mean(FM_ui(I_best_index,:),1); % best member of current iteration
    end
    
  if elitist > 0 
   index_sub = list(1:elitist, 1);   
   FM_pop(index_sub,:) = FM_ui(index_sub,:);
  else  
   FM_pop = FM_ui;
  end

  index_sub = list(1:subp,1);
  FM_subpop = FM_pop(index_sub,:);
 
 % store fitness evolution and obj fun evolution as well
    fprintf('Fitness value: %f\n',fitMaxVector(1,gen) )
    fprintf('Generation: %d\n',gen)
   
    gen=gen+1;
    fitMaxVector(1,gen) = sbest;
        

end %---end while ((I_iter < I_itermax) ...
Fit_and_p=[fitMaxVector(1,gen) 0]; %;p2;p3;p4]
FVr_bestmemit = update_eda(FVr_bestmemit,low_habitat_limit,up_habitat_limit,BRM); 


 
% VECTORIZED THE CODE INSTEAD OF USING FOR
function pop=genpop(a,b,lowMatrix,upMatrix)
 pop=unifrnd(lowMatrix,upMatrix,a,b);

% VECTORIZED THE CODE INSTEAD OF USING FOR
 function p=update_eda(p,lowMatrix,upMatrix,BRM)
  switch BRM
    case 1 % max and min replace
        [idx] = find(p<lowMatrix);
        p(idx)=lowMatrix(idx);
        [idx] = find(p>upMatrix);
        p(idx)=upMatrix(idx);
    case 2 %Random reinitialization
        [idx] = [find(p<lowMatrix);find(p>upMatrix)];
        replace=unifrnd(lowMatrix(idx),upMatrix(idx),length(idx),1);
        p(idx)=replace;
    case 3 %Bounce Back
        [idx] = find(p<lowMatrix);
      p(idx)=unifrnd(lowMatrix(idx),p(idx),length(idx),1);
        [idx] = find(p>upMatrix);
      p(idx)=unifrnd(p(idx), upMatrix(idx),length(idx),1);
   end


function pop_eda = learningEDA(pop_u,I_NP)
 %Cauchy's distribution
 mu = mean(pop_u);
 sd = std(pop_u);
for i=1:I_NP
  pop_eda(i,:) = normrnd(mu,sd).*(mu - sd*tan(pi*(rand(1,1))-0.5));
end
 