clc;clear all;close all;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GECAD GECCO and CEC 2020 Competition: Evolutionary Computation in Uncertain Environments: A Smart Grid Application 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath('Comp_functions') %Necessary functions to run the algorithms (encrypted)
tTotalTime=tic; % lets track total computational time

Select_testbed=2;
%Testbed 1: Energy resource management with uncertainty
%Testbed 2: Optimal bidding in local energy markets

Select_Algorithm=3;
%1: DE algorithm (test algorithm)
%6: Your algorithm

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Load MH parameters (e.g., get MH parameters from DEparameters.m file)
 switch Select_Algorithm
      case 3
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
         %Your algorithm can be inserted here
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          addpath('DEEDAalg')
          algorithm='DEEDAalg'; %'The participants should include their algorithm here'
          DEparameters %Function defined by the participant
          display('DE')
          No_solutions=deParameters.I_NP; %Notice that some algorithms are limited to one individual
       otherwise
         fprintf(1,'No algorithm selected\n');
 end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Load Data base 
noRuns=20; %this can be changed but final results should be based on 20 trials
DB=Select_testbed; 
% 1: Case study testbed 1
% 2: Case study testbed 2///
[caseStudyData, DB_name]=callDatabase(DB);

%% Label of the algorithm and the case study
Tag.algorithm=algorithm;
Tag.DB=DB_name;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Set other parameters
otherParameters =setOtherParameters(caseStudyData,No_solutions,Select_testbed);
 
% %% Set lower/upper bounds of variables 
[lowerB,upperB] = setVariablesBounds(caseStudyData,otherParameters, Select_testbed);
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Call the MH for optimizationclear 
ResDB=struc([]);
    for iRuns=1:noRuns %Number of trails
        tOpt=tic;
        rand('state',sum(noRuns*100*clock))% ensure stochastic indpt trials
        otherParameters.iRuns=iRuns;      
        
            switch Select_Algorithm
                case 3
                    [ResDB(iRuns).Fit_and_p, ...
                    ResDB(iRuns).sol, ...
                    ResDB(iRuns).fitVector]= ...                    
                    deopt_simple(deParameters,caseStudyData,otherParameters,lowerB,upperB);       
                otherwise
                  fprintf(1,'No algorithm selected\n');
             end
        ResDB(iRuns).tOpt=toc(tOpt); % time of each trial
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %% Save the results and stats
        Save_results
    end
tTotalTime=toc(tTotalTime); %Total time
%% End of MH Optimization

for j=1:noRuns
      lower_violations=ResDB(j).sol<lowerB;
      upper_violations=ResDB(j).sol>upperB;
      many_low(j)=sum(lower_violations);
      many_up(j)=sum(upper_violations);
      fprintf('lower_violations : %d .\n', many_low(j));
      fprintf('upper_violations : %d .\n', many_up(j));
end






